/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoComparisonFailure.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 * 
 */

#include <CocoUnit/CocoComparisonFailure.h>
#include <CocoUnit/CocoAssert.h>

@implementation CocoComparisonFailure

+(id) comparisonFailure:(NSString*) what
{
  return [ self comparisonFailure: what expected: nil actual: nil ];
}

+(id) comparisonFailure:(NSString*) what
	       expected:(NSString*) expected
		 actual:(NSString*) actual
{
  NSMutableDictionary* info = [ NSMutableDictionary dictionary ];
  [ info setObject: !expected ? (NSObject*)[ NSNull null ] : (NSObject*)expected
	 forKey: @"expected" ];
  [ info setObject: !actual ? (NSObject*)[ NSNull null ] : (NSObject*)actual
	 forKey: @"actual" ];
  return [ super exceptionWithName: [ self className ]
		 reason: what
		 userInfo: info ];
}

+(void) raise:(NSString*) what
     expected:(NSString*) expected
       actual:(NSString*) actual
{
  @throw [ self comparisonFailure: what expected: expected actual: actual ];
}

+(void) raise:(NSString*) what
{
  [ self raise: what expected: nil actual: nil ];
}

-(NSString*) reason
{
  NSDictionary* const info = [ self userInfo ];
  NSString* expected = [ info objectForKey: @"expected" ];
  NSString* actual = [ info objectForKey: @"actual" ];
  if ([ expected isMemberOfClass: [ NSNull class ] ]
      || [ actual isMemberOfClass: [ NSNull class ] ])
    return [ CocoAssert format: [ super reason ] expected: expected actual: actual ];
   // init expected			       
   const int elen = [ expected length ];	       
   unichar* const ebegin = calloc (elen + 1, sizeof (unichar));
   [ expected getCharacters: ebegin ];	       
   const unichar* ehead = ebegin;	       
   const unichar* eend = ebegin + elen;	       
   const unichar* etail = eend;		       
   // init actual			       
   const int alen = [ actual length ];	       
   unichar* const abegin = calloc (alen + 1, sizeof (unichar));
   [ actual getCharacters: abegin ];	       
   const unichar* ahead = abegin;	       
   const unichar* aend = abegin + alen;	       
   const unichar* atail = aend;		       
   // match head				       
   const unichar** limit = (elen < alen) ? &ehead : &ahead;
   while (**limit && *ehead == *ahead)	       
     ehead ++, ahead ++;			       
   // match tail				       
   for (;;) 
     if (*etail != *atail)		       
       {
	 etail ++, atail ++;
	 break;
       }
     else if (ehead < etail && ahead < atail)
       etail --, atail --;		       
     else
       break;
  if (ehead >= etail && ahead >= atail)
    return [ CocoAssert format: [ super reason ] expected: expected actual: actual ];
   NSMutableString* e = [ NSMutableString stringWithCapacity: etail - ehead + 7 ];
   NSString* const leader = @"..?";
   NSString* const trailer = @"?..";
   if (ebegin < ehead)
     [ e appendString: leader ];
   if (ehead < etail)
     [ e appendString:[NSString stringWithCharacters:ehead length: etail - ehead]];
   if (etail < eend)
     [ e appendString: trailer ];
   NSMutableString* a = [ NSMutableString stringWithCapacity: atail - ahead + 7 ];
   if (abegin < ahead)
     [ a appendString: leader ];
   if (ahead < atail)
     [ a appendString:[NSString stringWithCharacters:ahead length: atail - ahead]];
  if (atail < aend)
    [ a appendString: trailer ];
  free (ebegin);
  free (abegin);
  return [ CocoAssert format: [ super reason ] expected: e actual: a ];
}

@end
// Local variables: ***
// compile-command: "xcodebuild -project CocoUnit.xcode -target CocoUnitTest" ***
// End: ***
