/* Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestCase.m,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#include <CocoUnit/CocoTestCase.h>
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestSuite.h>
#include <objc/objc-class.h>
#include <objc/objc-runtime.h>

@implementation CocoTestCase

+(CocoTestSuite*)suite
{
  return [ CocoTestSuite testSuiteWithClass: [ self class ] ];
}

+(id) testCase
{
  return [ self testCaseWithName: [ self className ] ];
}

+(id) testCaseWithName:(NSString*) caseName
{
  return [ [ [ self alloc ] initWithName: caseName ] autorelease ];
}

-(id) init
{
  return [ self initWithName: [ self className ] ];
}

-(id) initWithName:(NSString*) caseName
{
  [ super init ];
  name = [ caseName retain ];
  return self;
}

-(void) dealloc
{
  [ name release ];
  [ super dealloc ];
}

-(int) countTestCases
{
  return 1;
}

/**
 * Creates a default CocoTestResult object
 *
 * @see CocoTestResult
 */
-(CocoTestResult*) createResult
{
  return [ CocoTestResult testResult ];
}

-(NSString*) name
{
  return name;
}

-(CocoTestResult*) run
{
  CocoTestResult* result = [ self createResult ];
  [ self runWithResult: result ];
  return result;
}

-(void) runWithResult:(CocoTestResult*) result
{
  [ result retain ];
  [ result runWithTest: self ];
  [ result release ];
}

-(void) runTest
{
  SEL selector = NSSelectorFromString (name);
  Method runMethod = NULL;
  if (selector)
    runMethod = class_getInstanceMethod ([ self class ], selector);
  if (!runMethod)
    [ self failWithFormat: @"Method \"%@\" not found", name ];
  objc_msgSend (self, selector);
}

-(void) setUp
{
}

-(void) tearDown
{
}

-(void) runBare
{
  [ self setUp ];
  @try 
  {
    [ self runTest ];
  }
  @finally 
  {
    [ self tearDown ];
  }
}

-(NSString*) description
{
  return [ NSString stringWithFormat: @"%@(%@)",
		    [ self name ],
		    [ self className ] ];
}

@end
