/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestFailure.h,v 1.1 2003/11/29 12:34:30 tsutsumi Exp $
 */

/** @file
 */

#ifndef COCOTESTFAILURE_H
#define COCOTESTFAILURE_H

#import <Foundation/NSObject.h>
@class CocoTest;

/**
 * A <code>CocoTestFailure</code> collects a failed test together with
 * the caught exception.
 * @see CocoTestResult
 */
@interface CocoTestFailure : NSObject {
  /** failed test object */
  CocoTest* failedTest;
  /** thrown exception */
  NSException* thrownException;
}

/**
 * Construct a TestFailure with the given test and exception.
 */
+(id) failureWithTest:(CocoTest*) test exception:(NSException*) exception;
-(id) init;
-(id) initWithTest:(CocoTest*) test exception:(NSException*) exception;

/**
 * Destruct a TestFailure.
 */
-(void) dealloc;

/**
 * Gets the failed test.
 */
-(id) failedTest;

/**
 * Gets the thrown exception.
 */
-(id) thrownException;

/**
 * Returns a short description of the failure.
 */
-(NSString*) description;

-(NSString*) exceptionMessage;

-(bool) isFailure;

@end

#endif // COCOTESTFAILURE_H
