/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestFailure.m,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#include <CocoUnit/CocoTestFailure.h>
#include <CocoUnit/CocoTest.h>
#include <CocoUnit/CocoAssertionFailedError.h>

@implementation CocoTestFailure

+(id) failureWithTest:(CocoTest*) test exception:(NSException*) exception
{
  return [ [ [ self alloc ] initWithTest: test exception: exception ]
	   autorelease ];
}

-(id) init
{
  return [ self initWithTest: nil exception: nil ];
}

-(id) initWithTest:(CocoTest*) test exception:(NSException*) exception
{
  [ super init ];
  failedTest = [ test retain ];
  thrownException = [ exception retain ];
  return self;
}

-(id) failedTest
{
  return failedTest;
}

-(id) thrownException
{
  return thrownException;
}

-(void) dealloc
{
  [ failedTest release ];
  [ thrownException release ];
  [ super dealloc ];
}

-(NSString*) description
{
  return [ NSString stringWithFormat: @"%@: %@",
		    [ failedTest description ],
		    [ thrownException reason ] ];
}

-(NSString*) exceptionMessage
{
  return [ thrownException reason ];
}

-(bool) isFailure
{
  return [ thrownException isKindOfClass: [ CocoAssertionFailedError class ] ];
}

@end
