/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the iKtai Library.

   The iKtai Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The iKtai Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the iKtai Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestSuite.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#include <CocoUnit/CocoTestSuite.h>
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTest.h>
#include <CocoUnit/CocoAssert.h>
#include <CocoUnit/CocoTestCase.h>
#include <CocoUnit/CocoTestWarning.h>
#import <objc/objc-runtime.h>

@implementation CocoTestSuite

+(id) testSuite
{
  return [ [ self new ] autorelease ];
}

+(id) testSuiteWithClass:(Class) testClass
{
  return [ [ [ self alloc ] initWithClass: testClass ] autorelease ];
}

+(id) testSuiteWithName:(NSString*) suiteName
{
  return [ [ [ self alloc ] initWithName: suiteName ] autorelease ];
}

-(id) init
{
  return [ self initWithName: nil ];
}

-(id) initWithName:(NSString*) suiteName
{
  [ super init ];
  tests = [ [ NSMutableArray alloc ] init ];
  name = [ suiteName retain ];
  return self;
}

-(id) initWithClass:(Class) testClass
{
  [ self initWithName: [ NSString stringWithCString: testClass->name ] ];
  Class superClass = testClass;
  NSMutableArray* names = [ [ NSMutableArray alloc ] init ];
  while ([ objc_getClass (superClass->name) isSubclassOfClass: [ CocoTest class ] ])
    {
      struct objc_method_list* list;
      void* iterator = NULL;
      while ((list = class_nextMethodList (superClass, &iterator)) != NULL)
	{
	  int i;
	  for (i = list->method_count - 1; i >= 0; i --)
	    [ self addTestMethod: &list->method_list[i]
		   names: names
		   class: superClass ];
	}
      superClass = superClass->super_class;
    }
  if ([ tests count ] == 0)
    [ self addTest: [ self warning: [ NSString stringWithFormat:
						 @"No tests fond in %s",
					       testClass->name ] ] ];
  [ names release ];
  return self;
}

-(void) dealloc
{
  [ name release ];
  [ tests release ];
  [ super dealloc ];
}

-(void) addTestMethod:(Method) method
		names:(NSMutableArray*) names
		class:(Class) class
{
  NSString* methodName;
  methodName = [ NSString stringWithCString: sel_getName (method->method_name) ];
  if ([ names containsObject: methodName ])
    return;
  if ([ self isTestMethod: method ])
    {
      CocoTestCase* test = nil;
      [ names addObject: methodName ];
      test = objc_msgSend (objc_getClass (class->name), @selector(alloc));
      [ tests addObject: [ [ test initWithName: methodName ] autorelease ] ];
    }
}

-(bool) isTestMethod:(Method) method
{
  if (!method)
    return false;
  const char* methodName = sel_getName (method->method_name);
  const char* methodTypes = method->method_types;
  unsigned numOfArgs = method_getNumberOfArguments (method);
  const char prefix[] = "test";
  return (numOfArgs == 2
	  && strncmp (methodName, prefix, sizeof (prefix) - 1) == 0
	  && methodTypes[0] == 'v');
}

-(void) addTest:(CocoTest*) test
{
  [ tests addObject: test ];
}

-(void) addTestSuite:(Class) testClass
{
  [ self addTest: [ CocoTestSuite testSuiteWithClass: testClass ] ];
}

-(int) countTestCases
{
  int count = 0;
  NSEnumerator* e = [ self tests ];
  id test;
  while ((test = [ e nextObject ]) != nil)
    {
      count += [ test countTestCases ];
    }
  return count;
}

-(void) runWithResult:(CocoTestResult*) result
{
  CocoTest* test;
  NSEnumerator* e = [ self tests ];
  while ((test = [ e nextObject ]) != nil)
    {
      NSAutoreleasePool* subpool = [ [ NSAutoreleasePool alloc ] init ];
      if ([ result shouldStop ])
        {
	  [ subpool release ];
	  break;
	}
      [ test runWithResult: result ];
      [ subpool release ];
    }
}

-(CocoTest*) testAt:(unsigned int) atIndex
{
  return [ tests objectAtIndex: atIndex ];
}

-(unsigned int) testCount
{
  return [ tests count ];
}

-(NSEnumerator*) tests
{
  return [ tests objectEnumerator ];
}

-(NSString*) description
{
  if (name != nil)
    return name;
  return [ super description ];
}

-(CocoTest*) warning:(NSString*) message
{
  return [ CocoTestWarning warning: message ];
}

@end
