/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the iKtai Library.

   The iKtai Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The iKtai Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the iKtai Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoBaseTestRunner.h,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#ifndef COCOBASETESTRUNNER_H
#define COCOBASETESTRUNNER_H

#include <CocoUnit/CocoTestListener.h>
#include <stdio.h>

@class CocoTestResult;

extern const int COCOUNIT_LISTENER_STATUS_ERROR;
extern const int COCOUNIT_LISTENER_STATUS_FAILURE;

/**
 * Abstract Base class for all test runners.
 */
@interface CocoBaseTestRunner : CocoTestListener {
}

+(id) testRunner;

/*
 * Implementation of Testlistener
 */
-(void) startTest:(CocoTest*) test;
-(void) endTest:(CocoTest*) test;
-(void) addError:(NSException*) error test:(CocoTest*) test;
-(void) addFailure:(CocoAssertionFailedError*) failure test:(CocoTest*) test;

/*
 * TestRunListener implementation
 */
-(void) testStarted:(NSString*) testName;
-(void) testEnded:(NSString*) testName;
-(void) testFailed:(int) status test:(CocoTest*) test exception:(NSException*) e;

/**
 * Returns the Test corresponding to the given suite. This is
 * a template method, subclasses override runFailed(), clearStatus().
 */
-(CocoTest*) testFromString:(NSString*) suiteClassName;

/**
 * Returns the formatted string of the elapsed time.
 */
-(NSString*) elapsedTimeAsString:(struct timeval*) runTime;

/**
 * Override to define how to handle a failed loading of a test suite.
 */
-(void) runFailed:(NSString*) message;

/**
 * Clears the status message.
 */
-(void) clearStatus;

@end

#endif // COCOBASETESTRUNNER_H
