/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: IMoney.h,v 1.2 2003/11/24 03:34:30 tsutsumi Exp $
 */

/** @file
 *
 */

#ifndef COCOUNIT_IMONEY_H
#define COCOUNIT_IMONEY_H

#import <Foundation/Foundation.h>

@class Money;
@class MoneyBag;

/**
 * The common interface for simple Monies and MoneyBags
 */
@interface IMoney : NSObject {

}

/**
 * Adds a money to this money.
 */
-(IMoney*) add:(IMoney*) m;

/**
 * Adds a simple Money to this money. This is a helper method for
 * implementing double dispatch
 */
-(IMoney*) addMoney:(Money*) m;

/**
 * Adds a MoneyBag to this money. This is a helper method for
 * implementing double dispatch
 */
-(IMoney*) addMoneyBag:(MoneyBag*) s;

/**
 * Tests whether this money is nil
 */
-(bool) isNil;

/**
 * Multiplies a money by the given factor.
 */
-(IMoney*) multiply:(int) factor;

/**
 * Negates this money.
 */
-(IMoney*) negate;

/**
 * Subtracts a money from this money.
 */
-(IMoney*) subtract:(IMoney*) m;

@end

#endif // COCOUNIT_IMONEY_H
