/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: Money.m,v 1.2 2003/11/24 03:34:30 tsutsumi Exp $
 */

/** @file
 * 
 */

#import "Money.h"

@implementation Money

+(id) money:(int) anAmount currency:(NSString*) aCurrency
{
  Money* newMoney = [ [ [ Money alloc ] init ] autorelease ];
  newMoney->amount = anAmount;
  newMoney->currency = [ aCurrency retain ];
  return newMoney;
}

-(void) dealloc
{
  [ currency release ];
  [ super dealloc ];
}

-(IMoney*) add:(IMoney*) m
{
  return [ m addMoney: self ];
}

-(IMoney*) addMoney:(Money*) m
{
  if ([ [ m currency ] isEqual: [ self currency ] ])
    return [ Money money: amount + [ m amount ] currency: currency ];
  return [ MoneyBag moneyBagWithMoney: self money: m ];
}

-(IMoney*) addMoneyBag:(MoneyBag*) s
{
  return [ s addMoney: self ];
}

-(int) amount
{
  return amount;
}

-(NSString*) currency
{
  return currency;
}

-(BOOL) isEqual:(id) anObject
{
  if ([ self isNil ])
    if ([ anObject isKindOfClass: [ IMoney class ]])
      return [ anObject isNil ];
  if ([ anObject isKindOfClass: [ Money class ] ])
    {
      Money* aMoney = anObject;
      return ([ [ aMoney currency ] isEqual: [ self currency ] ]
	      && [ self amount ] == [ aMoney amount ]);
    }
  return false;
}

-(unsigned) hash
{
  return [ currency hash ] + amount;
}

-(bool) isNil
{
  return amount == 0;
}

-(IMoney*) multiply:(int) factor
{
  return [ Money money: amount * factor currency: currency ];
}

-(IMoney*) negate
{
  return [ Money money: -amount currency: currency ];
}

-(IMoney*) subtract:(IMoney*) m
{
  return [ self add: [ m negate ] ];
}

-(NSString*) description
{
  return [ NSString stringWithFormat: @"[ %d %@ ]", amount, currency ];
}

@end
