/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: MoneyBag.h,v 1.2 2003/11/24 03:34:30 tsutsumi Exp $
 */

/** @file
 *
 */

#ifndef COCOUNIT_MONEYBAG_H
#define COCOUNIT_MONEYBAG_H

#include "IMoney.h"

/**
 * A MoneyBag defers exchange rate conversions. For example adding 
 * 12 Swiss Francs to 14 US Dollars is represented as a bag 
 * containing the two Monies 12 CHF and 14 USD. Adding another
 * 10 Swiss francs gives a bag with 22 CHF and 14 USD. Due to 
 * the deferred exchange rate conversion we can later value a 
 * MoneyBag with different exchange rates.
 *
 * A MoneyBag is represented as a list of Monies and provides 
 * different constructors to create a MoneyBag. 
 */
@interface MoneyBag : IMoney {
  NSMutableArray* monies;
}

+(id) moneyBagWithMonies:(Money**) bag count:(unsigned) count;
+(id) moneyBagWithMoney:(Money*) m1 money:(Money*) m2;
+(id) moneyBagWithMoney:(Money*) m bag:(MoneyBag*) bag;
+(id) moneyBagWithMoneyBag:(MoneyBag*) m1 bag:(MoneyBag*) m2;

-(void) appendMoneyBag:(MoneyBag*) aBag;
-(void) appendMoney:(Money*) aMoney;
-(bool) contains:(Money*) aMoney;
-(bool) isEqual:(NSObject*) anObject;
-(Money*) find:(NSString*) currency;
-(unsigned) hash;
-(bool) isNil;

@end

#endif // COCOUNIT_MONEYBAG_H
