/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: MoneyTest.m,v 1.5 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 *
 */

#include "MoneyTest.h"
#include <CocoUnit/CocoTextUiTestRunner.h>
#include <CocoUnit/CocoTestResult.h>

@implementation MoneyTest

-(void)setUp
{
  f1200JPY = [ Money money: 1200 currency: @"JPY" ];
  f1400JPY = [ Money money: 1400 currency: @"JPY" ];
  f7USD = [ Money money: 7 currency: @"USD" ];
  f21USD = [ Money money: 21 currency: @"USD" ];
  fMB1 = [ MoneyBag moneyBagWithMoney: f1200JPY money: f7USD ];
  fMB2 = [ MoneyBag moneyBagWithMoney: f1400JPY money: f21USD ];
}

-(void)tearDown
{
#if 0
  id objs[] = {
    f1200JPY,
    f1400JPY,
    f7USD,
    f21USD,
    fMB1,
    fMB2,
    nil,
  };
  int i;
  for (i = 0; objs[i] != nil; i ++)
    {
      NSLog(@"%@ : %d\n", objs[i], [ objs[i] retainCount ]);
    }
#endif
}

-(void) testBagMultiply
{
  // {[1200 JPY][7 USD]} *2 == {[2400 JPY][14 USD]}
  Money* bag[] = {
    [ Money money: 2400 currency: @"JPY" ],
    [ Money money: 14 currency: @"USD" ],
  };
  MoneyBag* expected = [ MoneyBag moneyBagWithMonies: bag count: 2 ];
  [ self assertEquals: expected actual: [ fMB1 multiply: 2 ] ]; 
  [ self assertTrue: [ [ fMB1 multiply: 0 ] isNil ] ];
}

-(void) testBagNegate
{
  // {[1200 JPY][7 USD]} negate == {[-1200 JPY][-7 USD]}
  Money* bag[]= {
    [ Money money: -1200 currency: @"JPY" ],
    [ Money money: -7 currency: @"USD" ],
  };
  MoneyBag* expected = [ MoneyBag moneyBagWithMonies: bag count: 2 ];
  [ self assertEquals: expected actual: [ fMB1 negate ] ];
}

-(void) testBagSimpleAdd
{
  // {[1200 JPY][7 USD]} + [1400 JPY] == {[26 JPY][7 USD]}
  Money* bag[] = {
    [ Money money: 2600 currency: @"JPY" ],
    [ Money money: 7 currency: @"USD" ],
  };
  MoneyBag* expected = [ MoneyBag moneyBagWithMonies: bag count: 2 ];
  [ self assertEquals: expected actual: [ fMB1 add: f1400JPY ] ];
}

-(void) testBagSubtract
{
  // {[1200 JPY][7 USD]} - {[1400 JPY][21 USD] == {[-200 JPY][-14 USD]}
  Money* bag[]= {
    [ Money money: -200 currency: @"JPY" ],
    [ Money money: -14 currency: @"USD" ]
  };
  MoneyBag* expected = [ MoneyBag moneyBagWithMonies: bag count: 2 ];
  [ self assertEquals: expected actual: [ fMB1 subtract: fMB2 ] ];
}

-(void) testBagSumAdd
{
  // {[1200 JPY][7 USD]} + {[1400 JPY][21 USD]} == {[2600 JPY][28 USD]}
  Money* bag[] = {
    [ Money money: 2600 currency: @"JPY" ],
    [ Money money: 28 currency: @"USD" ]
  };
  MoneyBag* expected = [ MoneyBag moneyBagWithMonies: bag count: 2 ];
  [ self assertEquals: expected actual: [ fMB1 add: fMB2 ] ];
}
-(void) testIsNil
{
  [ self assertTrue: [ [ fMB1 subtract: fMB1 ] isNil ]];
}
-(void) testMixedSimpleAdd
{
  // [1200 JPY] + [7 USD] == {[1200 JPY][7 USD]}
  Money* bag[]= { f1200JPY, f7USD };
  MoneyBag* expected = [ MoneyBag moneyBagWithMonies: bag count: 2 ];
  [ self assertEquals: expected actual: [ f1200JPY add: f7USD ] ];
}
-(void) testMoneyBagEquals
{
  [ self assertTrue: ![ fMB1 isEqual: nil ] ];
  [ self assertEquals: fMB1 actual: fMB1 ];
  MoneyBag* equal = [ MoneyBag moneyBagWithMoney:
				 [ Money money: 1200 currency: @"JPY" ]
			       money: [ Money money: 7 currency: @"USD" ] ];
  [ self assertTrue: [ fMB1 isEqual: equal ] ];
  [ self assertTrue: ! [ fMB1 isEqual: f1200JPY ] ];
  [ self assertTrue: ! [ f1200JPY isEqual: fMB1 ] ];
  [ self assertTrue: ! [ fMB1 isEqual: fMB2 ] ];
}

-(void) testMoneyBagHash
{
  MoneyBag* equal = [ MoneyBag moneyBagWithMoney:
				 [ Money money: 1200 currency: @"JPY" ]
			       money: [ Money money: 7 currency: @"USD" ] ];
  [ self assertEqualsLong: [ fMB1 hash ] actual: [ equal hash ] ];
}

-(void) testMoneyEquals
{
  [ self assertTrue: ! [ f1200JPY isEqual: nil ] ]; 
  Money* equalMoney= [ Money money: 1200 currency: @"JPY" ];
  [ self assertEquals: f1200JPY actual: f1200JPY ];
  [ self assertEquals: f1200JPY actual: equalMoney ];
  [ self assertEqualsLong: [ f1200JPY hash ] actual: [ equalMoney hash ] ];
  [ self assertTrue: ![ f1200JPY isEqual: f1400JPY ] ];
}
-(void) testMoneyHash
{
  [ self assertTrue: ![ f1200JPY isEqual: nil ] ]; 
  Money* equal= [ Money money: 1200 currency: @"JPY" ];
  [ self assertEqualsLong: [ f1200JPY hash ] actual: [ equal hash ] ];
}
-(void) testNormalize
{
  Money* bag[] = {
    [ Money money: 2600 currency: @"JPY" ],
    [ Money money: 2800 currency: @"JPY" ],
    [ Money money: 600 currency: @"JPY" ]
  };
  MoneyBag* moneyBag = [ MoneyBag moneyBagWithMonies: bag count: 3 ];
  Money* expected[] = { [ Money money: 6000 currency: @"JPY" ] };
  // note: expected is still a MoneyBag
  MoneyBag* expectedBag= [ MoneyBag moneyBagWithMonies: expected count: 1 ];
  [ self assertEquals: expectedBag actual: moneyBag ];
}

-(void) testNormalize2
{
  // {[1200 JPY][7 USD]} - [1200 JPY] == [7 USD]
  Money* expected = [ Money money: 7 currency: @"USD" ];
  [ self assertEquals: expected actual: [ fMB1 subtract: f1200JPY ] ];
}

-(void) testNormalize3
{
  // {[1200 JPY][7 USD]} - {[1200 JPY][3 USD]} == [4 USD]
  Money* s1[]= {
    [ Money money: 1200 currency: @"JPY" ],
    [ Money money: 3 currency: @"USD" ]
  };
  MoneyBag* ms1 = [ MoneyBag moneyBagWithMonies: s1 count: 2  ];
  Money* expected = [ Money money: 4 currency: @"USD" ];
  [ self assertEquals: expected actual: [ fMB1 subtract: ms1 ] ];
}
-(void) testNormalize4
{
  // [1200 JPY] - {[1200 JPY][3 USD]} == [-3 USD]
  Money* s1[]= {
    [ Money money: 1200 currency: @"JPY" ],
    [ Money money: 3 currency: @"USD" ]
  };
  MoneyBag* ms1 = [ MoneyBag moneyBagWithMonies: s1 count: 2 ];
  Money* expected = [ Money money: -3 currency: @"USD" ];
  [ self assertEquals: expected actual: [ f1200JPY subtract: ms1 ] ];
}

-(void) testPrint
{
  [ self assertEquals: @"[ 1200 JPY ]" actual: [ f1200JPY description ] ];
}

-(void) testSimpleAdd
{
  // [1200 JPY] + [1400 JPY] == [2600 JPY]
  Money* expected = [ Money money: 2600 currency: @"JPY" ];
  [ self assertEquals: expected actual: [ f1200JPY add: f1400JPY ] ];
}
-(void) testSimpleBagAdd
{
  // [1400 JPY] + {[1200 JPY][7 USD]} == {[2600 JPY][7 USD]}
  Money* bag[]= {
    [ Money money: 2600 currency: @"JPY" ],
    [ Money money: 7 currency: @"USD" ]
  };
  MoneyBag* expected = [ MoneyBag moneyBagWithMonies: bag count: 2 ];
  [ self assertEquals: expected actual: [ f1400JPY add: fMB1 ] ];
}

-(void) testSimpleMultiply
{
  // [1400 JPY] *2 == [2800 JPY]
  Money* expected = [ Money money: 2800 currency: @"JPY" ];
  [ self assertEquals: expected actual: [ f1400JPY multiply: 2 ] ];
}

-(void) testSimpleNegate
{
  // [1400 JPY] negate == [-1400 JPY]
  Money* expected = [ Money money: -1400 currency: @"JPY" ];
  [ self assertEquals: expected actual: [ f1400JPY negate ] ];
}
-(void) testSimpleSubtract
{
  // [1400 JPY] - [1200 JPY] == [200 JPY]
  Money* expected= [ Money money: 200 currency: @"JPY" ];
  [ self assertEquals: expected actual: [ f1400JPY subtract: f1200JPY ] ];
}

@end
