/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: AssertTest.m,v 1.4 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 *
 */

#include "AssertTest.h"
#include <CocoUnit/CocoAssertionFailedError.h>
#include <CocoUnit/CocoComparisonFailure.h>

inline id newObject ()
{
  return [ [ NSObject new ] autorelease ];
}

/**
 *
 */
@implementation AssertTest

/* In the tests that follow, we can't use standard formatting
 * for exception tests:
 *     try {
 *         somethingThatShouldThrow();
 *         fail();
 *     catch (AssertionFailedError e) {
 *     }
 * because fail() would never be reported.
 */

-(void) testFail
{
  // Also, we are testing fail, so we can't rely on fail() working.
  // We have to throw the exception manually, .
  @try
  {
    [ self fail ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ CocoAssertionFailedError raise ];
}

-(void) testAssertEquals
{
  NSObject* o = newObject ();
  [ self assertEquals: o actual: o ];
  @try
  {
    [ self assertEquals: newObject () actual: newObject () ];
  }
  @catch (CocoAssertionFailedError* e) 
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertEqualsNil
{
  [ self assertEquals: nil actual: nil ];
}

-(void) testAssertStringEquals
{
  [ self assertEqualsString: @"a" actual: @"a" ];
}

-(void) testAssertNilNotEqualsString
{
  @try
  {
    [ self assertEqualsString: nil actual: @"foo" ];
    [ self fail ];
  }
  @catch (CocoComparisonFailure* e)
  {
  }
}

-(void) testAssertStringNotEqualsNil
{
  @try
  {
    [ self assertEqualsString: @"foo" actual: nil ];
    [ self fail ];
  }
  @catch (CocoComparisonFailure* e)
  {
    [ e reason ]; // why no assertion?
  }
}

-(void) testAssertNilNotEqualsNil
{
  @try
  {
    [ self assertEquals: nil actual: newObject () ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    [ e reason ]; // why no assertion?
    return;
  }
  [ self fail ];
}

-(void) testAssertNil
{
  [ self assertNil: nil ];
  @try
  {
    [ self assertNil: newObject () ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertNotNil
{
  [ self assertNotNil: newObject () ];
  @try
  {
    [ self assertNotNil: nil ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertTrue
{
  [ self assertTrue: true ];
  @try 
  {
    [ self assertTrue: false ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertFalse
{
  [ self assertFalse: false ];
  @try
  {
    [ self assertFalse: true ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertSame
{
  NSObject* o = newObject ();
  [ self assertSame: o actual: o ];
  @try
  {
    [ self assertSame: [ NSNumber numberWithInt: 1 ]
	   actual: [ NSNumber numberWithInt: 1 ] ];
  }
  @catch (CocoAssertionFailedError*e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertNotSame
{
  [ self assertNotSame: [ NSNumber numberWithInt: 1 ] actual: nil ];
  [ self assertNotSame: nil actual: [ NSNumber numberWithInt: 1 ] ];
  [ self assertNotSame: [ NSNumber numberWithInt: 1 ]
	 actual: [ NSNumber numberWithInt: 1 ] ];
  @try
  {
    NSNumber* obj = [ NSNumber numberWithInt: 1 ];
    [ self assertNotSame: obj actual: obj ];
  }
  @catch (CocoAssertionFailedError*e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertNotSameFailsNil
{
  @try
  {
    [ self assertNotSame: nil actual: nil ];
  }
  @catch (CocoAssertionFailedError*e)
  {
    return;
  }
  [ self fail ];
}

@end
