/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: SuiteTest.m,v 1.5 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 *
 */

#include "FrameworkAllTests.h"
#include "SuiteTest.h"
#include "InheritedTestCase.h"
#include "NoTestCaseClass.h"
#include "NoTestCases.h"
#include "NotVoidTestCase.h"
#include "OneTestCase.h"
#include "OverrideTestCase.h"
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestSuite.h>
#include <CocoUnit/CocoTest.h>

@implementation SuiteTest

-(id)initWithName:(NSString*)caseName
{
  [ super initWithName: caseName ];
  fResult = nil;
  return self;
}

-(void) setUp
{
  fResult = [ CocoTestResult testResult ];
}

+(CocoTest*) suite
{
  CocoTestSuite* suite = [ CocoTestSuite testSuiteWithName: @"Suite Tests" ];
  // build the suite manually, because some of the suites are testing
  // the functionality that automatically builds suites
  [ suite addTest: [ SuiteTest testCaseWithName: @"testNoTestCaseClass" ] ];
  [ suite addTest: [ SuiteTest testCaseWithName: @"testNoTestCases" ] ];
  [ suite addTest: [ SuiteTest testCaseWithName: @"testOneTestCase" ] ];
  [ suite addTest: [ SuiteTest testCaseWithName: @"testNotVoidTestCase" ] ];
  [ suite addTest: [ SuiteTest testCaseWithName: @"testNotExistingTestCase" ] ];
  [ suite addTest: [ SuiteTest testCaseWithName: @"testInheritedTests" ] ];
  [ suite addTest: [ SuiteTest testCaseWithName: @"testShadowedTests" ] ];
  [ suite addTest: [ SuiteTest testCaseWithName: @"testAddTestSuite" ] ];
  return suite;
}

-(void) testInheritedTests
{
  CocoTestSuite* suite;
  suite = [ CocoTestSuite testSuiteWithClass: [ InheritedTestCase class ] ];
  [ suite runWithResult: fResult ];
  [ self assertTrue: [ fResult wasSuccessful ] ];
  [ self assertEqualsLong: 2 actual: [ fResult runCount ] ];
}

-(void) testNoTestCaseClass
{
  CocoTest* t = [ CocoTestSuite testSuiteWithClass: [ NoTestCaseClass class ] ];
  [ t runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: [ fResult runCount ] ]; // warning test
  [ self assertFalse: [ fResult wasSuccessful ] ];
}

-(void) testNoTestCases
{
  CocoTest* t = [ CocoTestSuite testSuiteWithClass: [ NoTestCases class ] ];
  [ t runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: [ fResult runCount ] ]; // warning test
  [ self assertEqualsLong: 1 actual: [ fResult failureCount ] ];
  [ self assertFalse: [ fResult wasSuccessful ] ];
}

-(void) testNotExistingTestCase
{
  CocoTest* t = [ SuiteTest testCaseWithName: @"notExistingMethod" ];
  [ t runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: [ fResult runCount ] ];
  [ self assertEqualsLong: 1 actual: [ fResult failureCount ] ];
  [ self assertEqualsLong: 0 actual: [ fResult errorCount ] ];
}

-(void) testNotVoidTestCase
{
  CocoTestSuite* suite;
  suite = [ CocoTestSuite testSuiteWithClass: [ NotVoidTestCase class ] ];
  [ self assertEqualsLong: 1 actual: [ suite countTestCases ] ];
}

-(void) testOneTestCase
{
  CocoTest* t = [ CocoTestSuite testSuiteWithClass: [ OneTestCase class ] ];
  [ t runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: [ fResult runCount ] ];
  [ self assertEqualsLong: 0 actual: [ fResult failureCount ] ];
  [ self assertEqualsLong: 0 actual: [ fResult errorCount ] ];
  [ self assertTrue: [ fResult wasSuccessful ] ];
}

-(void) testShadowedTests
{
  CocoTestSuite* suite;
  suite = [ CocoTestSuite testSuiteWithClass: [ OverrideTestCase class ] ];
  [ suite runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: [ fResult runCount ] ];
}

-(void) testAddTestSuite
{
  CocoTestSuite* suite;
  suite = [ CocoTestSuite testSuiteWithName: [ self className ] ];
  [ suite addTestSuite: [ OneTestCase class ] ];
  [ suite runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: [ fResult runCount ] ];
}

@end

// Local variables: ***
// compile-command: "xcodebuild -project ../../CocoUnit.xcode -target CocoUnitTest" ***
// End: ***
