/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: TestImplementorTest.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 * 
 */

#include "TestImplementorTest.h"
#include <CocoUnit/CocoProtectable.h>
#include <CocoUnit/CocoTestResult.h>

@interface DoubleProtectable : CocoProtectable
{
}
@end
@implementation DoubleProtectable
-(void) protect
{
  [ test runBare ];
  [ test runBare ];
}
@end

@interface DoubleTestCase : CocoTest
{
  CocoTestCase* testCase;
}
@end
@implementation DoubleTestCase
+(id) doubleTestCaseWithTestCase:(CocoTestCase*)testCase
{
#pragma unused (testCase)
  return [ [ self new ] autorelease ];
}

-(id) initWithTestCase:(CocoTestCase*)test
{
  self->testCase = test;
  return self;
}

-(int) countTestCases
{
  return 2;
}

-(void) runWithResult:(CocoTestResult*) result
{
  [ result startTest: self ];
  CocoProtectable* p = [ [ DoubleProtectable alloc ] initWithTest: testCase ];
  [ result runProtected: p ];
  [ p release ];
  [ result endTest: self ];
}
@end

@interface TestCase : CocoTestCase
{
}
@end
@implementation TestCase
-(void) runTest
{
}
@end

@implementation TestImplementorTest

-(id) initWithName:(NSString*) testName
{
  [ super initWithName: testName ];
  test = [ [ DoubleTestCase alloc ] initWithTestCase: [ TestCase testCase ] ];
  return self;
}

-(void) testSuccessfulRun
{
  CocoTestResult* testResult = [ CocoTestResult testResult ];
  [ test runWithResult: testResult ];
  [ self assertEqualsLong: [ test countTestCases ] actual: [ testResult runCount ] ];
  [ self assertEqualsLong: 0 actual: [ testResult errorCount ] ];
  [ self assertEqualsLong: 0 actual: [ testResult failureCount ] ];
}

@end
