/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: TestListenerTest.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 * 
 */

#include "TestListenerTest.h"
#include <CocoUnit/CocoError.h>
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestFailure.h>
#include <CocoUnit/CocoTestSuite.h>
#include <CocoUnit/CocoTestListener.h>
#import <Foundation/NSException.h>

@interface NoOpError : CocoTestCase {} @end
@implementation NoOpError
-(void) runTest
{
  [ self error ];
}
@end
@interface NoopFail : CocoTestCase {} @end
@implementation NoopFail
-(void) runTest
{
  [ self fail ];
}
@end
@interface Noop : CocoTestCase {} @end
@implementation Noop
-(void) runTest
{
}
@end

@interface TestListener : CocoTestListener {
 @public
  int fStartCount;
  int fEndCount;
  int fFailureCount;
  int fErrorCount;
}
-(void) setUp;
@end
@implementation TestListener
-(void) setUp
{
  fStartCount = 0;
  fEndCount = 0;
  fFailureCount = 0;
  fErrorCount = 0;
}

-(void) startTest:(CocoTest*) test
{
  fStartCount ++;
}

-(void) endTest:(CocoTest*) test
{
  fEndCount ++;
}

-(void) addTest:(CocoTest*) test error:(NSException*) e
{
  fErrorCount ++;
}

-(void) addTest:(CocoTest*) test failure:(CocoAssertionFailedError*) f
{
  fFailureCount ++;
}

@end

@implementation TestListenerTest

-(id) initWithName:(NSString*) caseName
{
  [ super initWithName: caseName ];
  fListener = [ TestListener listener ];
  return self;
}

-(void) setUp
{
  fResult = [ CocoTestResult testResult ];
  [ fResult addListener: fListener ];
  [ fListener setUp ];
}

-(void) testError
{
  CocoTestCase* test = [ NoOpError testCase ];
  [ test runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: fListener->fErrorCount ];
  [ self assertEqualsLong: 1 actual: fListener->fEndCount ];
}

-(void) testFailuer
{
  CocoTestCase* test = [ NoopFail testCase ];
  [ test runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: fListener->fFailureCount ];
  [ self assertEqualsLong: 1 actual: fListener->fEndCount ];
}

-(void) testStartStop
{
  CocoTestCase* test = [ Noop testCase ];
  [ test runWithResult: fResult ];
  [ self assertEqualsLong: 1 actual: fListener->fStartCount ];
  [ self assertEqualsLong: 1 actual: fListener->fEndCount ];
}

@end

// Local variables: ***
// compile-command: "xcodebuild -project ../../CocoUnit.xcode -target CocoUnitTest" ***
// End: ***
