/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: TextFeedbackTest.m,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 * 
 */

#include "TextFeedbackTest.h"
#include <CocoUnit/CocoTextUiResultPrinter.h>
#include <CocoUnit/CocoTextUiTestRunner.h>
#include <sys/time.h>

@interface TestResultPrinter : CocoTextUiResultPrinter
{
}
@end
@implementation TestResultPrinter
/* Spoof printing time so the tests are deterministic
 */
-(NSString*) elapsedTimeAsString:(struct timeval*) runTime
{
  return @"0";
}
@end

@interface TestCase : CocoTestCase
{
}
@end
@implementation TestCase
-(void) runTest
{
}
@end

@interface FailResultPrinter : TestResultPrinter
{
}
@end
@implementation FailResultPrinter
-(void) printFailures:(CocoTestResult*) result
{
  [ [ self writer ] printFormat: @"Failures here" ];
}
@end

@interface ErrorResultPrinter : TestResultPrinter
{
}
@end
@implementation ErrorResultPrinter
-(void) printErrors:(CocoTestResult*) result
{
  [ [ self writer ] printFormat: @"Errors here" ];
}
@end

@interface FailTestCase : CocoTestCase
{
}
@end
@implementation FailTestCase
-(void) runTest
{
  [ CocoAssertionFailedError raise ];
}
@end

@interface ErrorTestCase : CocoTestCase
{
}
@end
@implementation ErrorTestCase
-(void) runTest
{
  [ CocoError raise ];
}
@end

@implementation TextFeedbackTest : CocoTestCase

-(void) setUp
{
  output = [ CocoStringWriter stringWriter ];
  runner = [ CocoTextUiTestRunner testRunnerWithPrinter:
				    [ TestResultPrinter resultPrinter: output ] ];
}

-(void) testEmptySuite
{
  NSString* expected;
  expected = @"\nTime: 0\n\nOK (0 tests)\n\n";
  [ runner doRun: [ CocoTestSuite testSuite ] ];
  [ self assertEqualsString: expected actual: [ output description ] ];
}

-(void) testOneTest
{
  NSString* expected = @".\nTime: 0\n\nOK (1 test)\n\n";
  CocoTestSuite* suite = [ CocoTestSuite testSuite ];
  [ suite addTest: [ TestCase testCase ] ];
  [ runner doRun: suite ];
  [ self assertEqualsString: expected actual: [ output description ] ];
}

-(void) testTwoTests
{
  NSString* expected = @"..\nTime: 0\n\nOK (2 tests)\n\n";
  CocoTestSuite* suite = [ CocoTestSuite testSuite ];
  [ suite addTest: [ TestCase testCase ] ];
  [ suite addTest: [ TestCase testCase ] ];
  [ runner doRun: suite ];
  [ self assertEqualsString: expected actual: [ output description ] ];
}

-(void) testFailure
{
  NSString* expected = (@".F\nTime: 0\nFailures here\nFAILURES!!!\n"
			@"Tests run: 1, Failures: 1, Errors: 0\n\n");
  TestResultPrinter* printer;
  printer = [ FailResultPrinter resultPrinter: output ];
  [ runner printer: printer ];
  CocoTestSuite* suite = [ CocoTestSuite testSuite ];
  [ suite addTest: [ FailTestCase testCase ] ];
  [ runner doRun: suite ];
  [ self assertEqualsString: expected actual: [ output description ] ];
}

-(void) testError
{
  NSString* expected = ( @".E\nTime: 0\nErrors here\nFAILURES!!!\n"
			 @"Tests run: 1, Failures: 0, Errors: 1\n\n" );
  TestResultPrinter* printer;
  printer = [ ErrorResultPrinter resultPrinter: output ];
  [ runner printer: printer ];
  CocoTestSuite* suite = [ CocoTestSuite testSuite ];
  [ suite addTest: [ ErrorTestCase testCase ] ];
  [ runner doRun: suite ];
  [ self assertEqualsString: expected actual: [ output description ] ];
}
@end
