/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the iKtai Library.

   The iKtai Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The iKtai Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the iKtai Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTextUiTestRunner.h,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#ifndef COCOTEXTUITESTRUNNER_H
#define COCOTEXTUITESTRUNNER_H

#include <CocoUnit/CocoBaseTestRunner.h>
#include <stdio.h>

@class CocoTestResult;
@class CocoWriter;
@class CocoTextUiResultPrinter;

/**
 * A command line based tool to run tests.
 * <pre>
 * CocoTUTestRunner runWithClassName: TestCaseClass;
 * </pre>
 * TestRunner expects the name of a CocoTestCase class as argument.
 * If this class defines a static <code>suite</code> method it 
 * will be invoked and the returned test is run. Otherwise all 
 * the methods starting with "test" having no arguments are run.
 * <p>
 * When the wait command line argument is given TestRunner
 * waits until the users types RETURN.
 * <p>
 * TestRunner prints a trace as the tests are executed followed by a
 * summary at the end. 
 */
@interface CocoTextUiTestRunner : CocoBaseTestRunner {
  CocoTextUiResultPrinter* printer;
  bool verbose;
 @public
  enum
  {
    SUCCESS_EXIT,
    FAILURE_EXIT,
    EXCEPTION_EXIT,
  };
}

/**
 * Constructs a TestRunner.
 */
+(id) testRunner;

/**
 * Constructs a TestRunner using the given stream for all the output
 */
+(id) testRunnerWithStream:(FILE*) stream;

/**
 * Constructs a TestRunner using the given stream for all the output
 */
+(id) testRunnerWithWriter:(CocoWriter*) aWriter;

/**
 * Constructs a TestRunner using the given ResultPrinter all the output
 */
+(id) testRunnerWithPrinter:(CocoTextUiResultPrinter*) resultPrinter;

/**
 * Runs a suite extracted from a TestCase subclass.
 */
+(void) run:(Class) testClass;

/**
 * Runs a single test and collects its results.
 * This method can be used to start a test run
 * from your program.
 * <pre>
 * int main () {
 *     [ CocoTextUiTestRunner runWithSuite: suite() ];
 * }
 * </pre>
 */
+(CocoTestResult*) runWithSuite:(CocoTest*) suite;

-(id) init;
-(id) initWithStream:(FILE*) stream;
-(id) initWithWriter:(CocoWriter*) aWriter;
-(id) initWithPrinter:(CocoTextUiResultPrinter*) aPrinter;
-(CocoTestResult*) doRun:(CocoTest*) suite;

-(void) printer:(CocoTextUiResultPrinter*) printer;
-(void) verbose:(bool) isEnable;
@end

#endif // COCOTEXTUITESTRUNNER_H
